require 'dmao/api/entity'
require 'dmao/api/errors/invalid_dataset'
require 'dmao/api/errors/invalid_dataset_id'
require 'dmao/api/errors/invalid_project_id'
require 'dmao/api/errors/invalid_organisation_unit_id'
require 'dmao/api/errors/dataset_not_found'

module DMAO
  module API

    class Dataset < Entity

      ENDPOINT = 'datasets'
      NOT_FOUND_ERROR = DMAO::API::Errors::DatasetNotFound
      INVALID_ID_ERROR = DMAO::API::Errors::InvalidDatasetID
      INVALID_ENTITY_CLASS = DMAO::API::Errors::InvalidDataset
      INVALID_ENTITY_ERROR_MESSAGE = "Invalid dataset details, please see errors."

      VALID_ATTRIBUTES = [
          :id,
          :institution_id,
          :permanent_id,
          :title,
          :description,
          :publisher,
          :access,
          :notes,
          :creation_date,
          :system_uuid,
          :system_modified_at,
          :project_id,
          :owning_unit_id
      ]

      attr_reader(*VALID_ATTRIBUTES)

      def initialize(attributes)

        @id = attributes[:id]
        @institution_id = attributes[:institution_id]
        @permanent_id = attributes[:permanent_id]
        @title = attributes[:title]
        @description = attributes[:description]
        @publisher = attributes[:publisher]
        @access = attributes[:access]
        @notes = attributes[:notes]
        @creation_date = attributes[:creation_date]
        @system_uuid = attributes[:system_uuid]
        @system_modified_at = attributes[:system_modified_at]
        @project_id = attributes[:project_id]
        @owning_unit_id = attributes[:owning_unit_id]

      end

      def self.instance_from_api_data data

        attributes = {
            id: data["id"],
            institution_id: data["relationships"]["institution"]["data"]["id"],
            permanent_id: data["attributes"]["permanent-id"],
            title: data["attributes"]["title"],
            description: data["attributes"]["description"],
            publisher: data["attributes"]["publisher"],
            access: data["attributes"]["access"],
            notes: data["attributes"]["notes"],
            creation_date: data["attributes"]["creation-date"],
            system_uuid: data["attributes"]["system-uuid"],
            system_modified_at: data["attributes"]["system-modified-at"],
            project_id: data["relationships"]["project"]["data"]["id"],
            owning_unit_id: data["relationships"]["owning-unit"]["data"]["id"]
        }

        new(attributes)

      end

      def self.handle_unprocessable_entity error_response

        errors = parse_error_response error_response

        raise_error_if_key DMAO::API::Errors::InvalidProjectID, errors, "project"
        raise_error_if_key DMAO::API::Errors::InvalidOrganisationUnitID, errors, "owning_unit"

        raise DMAO::API::Errors::InvalidDataset.new("Invalid dataset details, please see errors.", errors)


      end

    end

  end
end