require 'rest-client'
require 'dmao/api/entity'
require 'dmao/api/errors/invalid_parent_id'
require 'dmao/api/errors/invalid_organisation_unit'
require 'dmao/api/errors/invalid_organisation_unit_id'
require 'dmao/api/errors/organisation_unit_not_found'

module DMAO
  module API

    class OrganisationUnit < Entity

      ENDPOINT = 'organisation_units'
      NOT_FOUND_ERROR = DMAO::API::Errors::OrganisationUnitNotFound
      INVALID_ID_ERROR = DMAO::API::Errors::InvalidOrganisationUnitID
      INVALID_ENTITY_CLASS = DMAO::API::Errors::InvalidOrganisationUnit
      INVALID_ENTITY_ERROR_MESSAGE = "Invalid person details, please see errors."

      VALID_ATTRIBUTES = [:id, :institution_id, :name, :description, :url, :system_uuid, :system_modified_at, :isni, :unit_type, :parent_id]
      attr_reader(*VALID_ATTRIBUTES)

      def initialize(attributes)

        @id = attributes[:id]
        @institution_id = attributes[:institution_id]
        @name = attributes[:name]
        @description = attributes[:description]
        @url = attributes[:url]
        @system_uuid = attributes[:system_uuid]
        @system_modified_at = attributes[:system_modified_at]
        @isni = attributes[:isni]
        @unit_type = attributes[:unit_type]

        if attributes[:parent_id]
          @parent_id = attributes[:parent_id]
        end

      end

      def self.instance_from_api_data data

        parent_id = set_id_attribute_if_not_nil data["relationships"]["parent"]["data"]

        attributes = {
            id: data["id"],
            institution_id: data["relationships"]["institution"]["data"]["id"],
            name: data["attributes"]["name"],
            description: data["attributes"]["description"],
            url: data["attributes"]["url"],
            system_uuid: data["attributes"]["system-uuid"],
            system_modified_at: data["attributes"]["system-modified-at"],
            isni: data["attributes"]["isni"],
            unit_type: data["attributes"]["unit-type"],
            parent_id: parent_id
        }

        new(attributes)

      end

      def self.handle_unprocessable_entity error_response

        errors = parse_error_response error_response

        raise_error_if_key DMAO::API::Errors::InvalidParentId, errors, "parent"

        raise DMAO::API::Errors::InvalidOrganisationUnit.new("Invalid organisation unit details, please see errors.", errors)

      end

    end

  end
end