require 'dmao/api/entity'
require 'dmao/api/errors/invalid_project_participant'
require 'dmao/api/errors/invalid_project_participant_id'
require 'dmao/api/errors/project_participant_not_found'
require 'dmao/api/errors/invalid_project_id'
require 'dmao/api/errors/invalid_person_id'
require 'dmao/api/errors/unsupported_query_by_system_uuid'

module DMAO
  module API

    class ProjectParticipant < Entity

      ENDPOINT = 'project_participants'
      NOT_FOUND_ERROR = DMAO::API::Errors::ProjectParticipantNotFound
      INVALID_ID_ERROR = DMAO::API::Errors::InvalidProjectParticipantID
      INVALID_ENTITY_CLASS = DMAO::API::Errors::InvalidProjectParticipant
      INVALID_ENTITY_ERROR_MESSAGE = "Invalid project participant details, please see errors."

      VALID_ATTRIBUTES = [:id, :institution_id, :role, :project_id, :participant_id, :person_id]

      attr_reader(*VALID_ATTRIBUTES)

      def initialize(attributes)

        @id = attributes[:id]
        @institution_id = attributes[:institution_id]
        @role = attributes[:role]
        @project_id = attributes[:project_id]
        @participant_id = @person_id = attributes[:participant_id]

      end

      def self.instance_from_api_data data

        attributes = {
            id: data["id"],
            institution_id: data["relationships"]["institution"]["data"]["id"],
            role: data["attributes"]["role"],
            project_id: data["relationships"]["project"]["data"]["id"],
            participant_id: data["relationships"]["person"]["data"]["id"]
        }

        new(attributes)

      end

      def self.handle_unprocessable_entity error_response

        errors = parse_error_response error_response

        raise DMAO::API::Errors::InvalidProjectParticipant.new("Invalid project participant details, please see errors.", errors) if (errors.keys.include?("project") && errors.keys.include?("person"))

        raise_error_if_key DMAO::API::Errors::InvalidProjectID, errors, "project"
        raise_error_if_key DMAO::API::Errors::InvalidPersonID, errors, "person"

        raise DMAO::API::Errors::InvalidProjectParticipant.new("Invalid project participant details, please see errors.", errors)


      end

      def self.find_by_system_uuid _system_uuid
        raise DMAO::API::Errors::UnsupportedQueryBySystemUUID.new
      end

      def self.find_project_participant project_id, participant_id

        response = self.api["#{self::ENDPOINT}?project=#{project_id}&person=#{participant_id}"].get

        response_data = JSON.parse(response)["data"]

        raise self::NOT_FOUND_ERROR.new if response_data.length == 0
        raise DMAO::API::Errors::InvalidResponseLength.new("Expected 1 element in response there were #{response_data.length}") if response_data.length != 1

        data = response_data[0]

        instance_from_api_data data

      end

    end

  end
end