require 'dmao/api/entity'
require 'dmao/api/errors/invalid_publication'
require 'dmao/api/errors/invalid_publication_id'
require 'dmao/api/errors/invalid_project_id'
require 'dmao/api/errors/invalid_organisation_unit_id'
require 'dmao/api/errors/publication_not_found'

module DMAO
  module API

    class Publication < Entity

      ENDPOINT = 'publications'
      NOT_FOUND_ERROR = DMAO::API::Errors::PublicationNotFound
      INVALID_ID_ERROR = DMAO::API::Errors::InvalidPublicationID
      INVALID_ENTITY_CLASS = DMAO::API::Errors::InvalidPublication
      INVALID_ENTITY_ERROR_MESSAGE = "Invalid publication details, please see errors."

      VALID_ATTRIBUTES = [
          :id,
          :institution_id,
          :permanent_id,
          :title,
          :abstract,
          :category,
          :publication_type,
          :status,
          :url,
          :publication_date,
          :system_uuid,
          :system_modified_at,
          :project_id,
          :owning_unit_id
      ]

      attr_reader(*VALID_ATTRIBUTES)

      def initialize(attributes)

        @id = attributes[:id]
        @institution_id = attributes[:institution_id]
        @permanent_id = attributes[:permanent_id]
        @title = attributes[:title]
        @abstract = attributes[:abstract]
        @category = attributes[:category]
        @publication_type = attributes[:publication_type]
        @status = attributes[:status]
        @url = attributes[:url]
        @publication_date = attributes[:publication_date]
        @system_uuid = attributes[:system_uuid]
        @system_modified_at = attributes[:system_modified_at]
        @project_id = attributes[:project_id]
        @owning_unit_id = attributes[:owning_unit_id]

      end

      def self.instance_from_api_data data

        attributes = {
            id: data["id"],
            institution_id: data["relationships"]["institution"]["data"]["id"],
            permanent_id: data["attributes"]["permanent-id"],
            title: data["attributes"]["title"],
            abstract: data["attributes"]["abstract"],
            category: data["attributes"]["category"],
            publication_type: data["attributes"]["publication-type"],
            status: data["attributes"]["status"],
            url: data["attributes"]["url"],
            publication_date: data["attributes"]["publication-date"],
            system_uuid: data["attributes"]["system-uuid"],
            system_modified_at: data["attributes"]["system-modified-at"],
            project_id: data["relationships"]["project"]["data"]["id"],
            owning_unit_id: data["relationships"]["owning-unit"]["data"]["id"]
        }

        new(attributes)

      end

      def self.handle_unprocessable_entity error_response

        errors = parse_error_response error_response

        raise_error_if_key DMAO::API::Errors::InvalidProjectID, errors, "project"
        raise_error_if_key DMAO::API::Errors::InvalidOrganisationUnitID, errors, "owning_unit"

        raise DMAO::API::Errors::InvalidPublication.new("Invalid Publication details, please see errors.", errors)


      end

    end

  end
end